@@ -19,12 +19,29 @@
 //! ``` 
 //!
 //! The `ArrayLength<T>` trait is implemented by default for [unsigned integer types](../typenum/uint/index.html) from [typenum](../typenum/index.html).
+//!
+//! For ease of use, an `arr!` macro is provided - example below:
+//!
+//! ```
+//! # #[macro_use] 
+//! # extern crate generic_array;
+//! # extern crate typenum;
+//! # fn main() {
+//! let array = arr![u32; 1, 2, 3];
+//! assert_eq!(array[2], 3);
+//! # }
+//! ```
+#![cfg_attr(feature="no_std", no_std)]
+#[cfg(feature="no_std")]
+extern crate core as std;
 extern crate typenum;
+pub mod arr;
 use typenum::uint::{Unsigned, UTerm, UInt};
 use typenum::bit::{B0, B1};
 use std::marker::PhantomData;
 use std::mem;
 use std::ops::{Deref, DerefMut};
+use std::ptr;
 use std::slice;
 
 /// Trait making GenericArray work, marking types to be used as length of an array
@@ -115,9 +132,11 @@ impl<T: Default, N> GenericArray<T, N> where N: ArrayLength<T> {
 
     /// Function constructing an array filled with default values
     pub fn new() -> GenericArray<T, N> {
-        let mut res: GenericArray<T, N> = unsafe { mem::zeroed() };
-        for r in res.iter_mut() { *r = T::default(); }
-        res
+        unsafe {
+            let mut res: GenericArray<T, N> = mem::uninitialized();
+            for r in res.iter_mut() { ptr::write(r, T::default()) }
+            res
+        }
     }
 
 }
@@ -127,20 +146,22 @@ impl<T: Clone, N> GenericArray<T, N> where N: ArrayLength<T> {
     /// Function constructing an array from a slice; the length of the slice must be equal to the length of the array
     pub fn from_slice(list: &[T]) -> GenericArray<T, N> {
         assert_eq!(list.len(), N::to_usize());
-        let mut res: GenericArray<T, N> = unsafe { mem::zeroed() };
-        for i in 0..N::to_usize() {
-            res[i] = list[i].clone();
+        unsafe {
+            let mut res: GenericArray<T, N> = mem::uninitialized();
+            for i in 0..N::to_usize() { ptr::write(&mut res[i], list[i].clone()) }
+            res
         }
-        res
     }
 
 }
 
 impl<T: Clone, N> Clone for GenericArray<T, N> where N: ArrayLength<T> {
     fn clone(&self) -> GenericArray<T, N> {
-        let mut res: GenericArray<T, N> = unsafe { mem::zeroed() };
-        for i in 0..N::to_usize() { res[i] = self[i].clone(); }
-        res
+        unsafe {
+            let mut res: GenericArray<T, N> = mem::uninitialized();
+            for i in 0..N::to_usize() { ptr::write(&mut res[i], self[i].clone()) }
+            res
+        }
     }
 }
 impl<T: Copy, N> Copy for GenericArray<T, N> where N: ArrayLength<T>, N::ArrayType: Copy {}
